type Fn = {
    name: string;
    fn: (...args: any) => any;
};
type CallContext = {
    that: unknown;
    args: IArguments;
};
type ExecuteState = {
    state: 'execute';
};
type ErrorState = {
    state: 'error';
    error: unknown;
};
type NormalState = {
    state: 'result';
    result: unknown;
};
type InstrumentHandler = {
    name?: string;
    onExecute?: (context: Fn & CallContext & ExecuteState) => void | Partial<CallContext & Fn & ExecuteState>;
    onError?: (context: Fn & CallContext & ErrorState & {
        isPromise: boolean;
    }) => void | ErrorState | NormalState;
    onResult?: (context: Fn & CallContext & NormalState & {
        isPromise: boolean;
    }) => void | ErrorState | NormalState;
};
type CreateInstrumentHandler = InstrumentHandler | (() => InstrumentHandler);
declare const createInstrument: (opts: CreateInstrumentHandler) => InstrumentHandler;
type InstrumentFunction = <X extends Fn['fn']>(fn: X, name?: string) => X;
declare function composeInstrument(...ins: CreateInstrumentHandler[]): InstrumentFunction | undefined;

type Provider<Provide, Input = unknown> = (() => Provide | Promise<Provide>) | ((input: Input) => Provide | Promise<Provide>);
type ProviderOption = {
    name?: string;
    instrument?: InstrumentFunction;
    mode?: 'prototype' | 'singleton';
};
declare const defaultProviderOptions: ProviderOption;
declare function setInstrument(inst: CreateInstrumentHandler): void;
type Executor<Provide> = {
    execute<Output>(executable: Provider<Output, Provide>): Promise<Output>;
    get(): Promise<Provide>;
    prepare<Input extends Array<any>, Output>(provider: (provide: Provide, ...input: Input) => Output): (...input: Input) => Promise<Awaited<ReturnType<typeof provider>>>;
};
type Hijacked<Dependent> = (executor: Executor<Dependent>) => void;
declare function create<Provide, Dependent = unknown>(provider: Provider<Provide, Dependent>, dependent?: Executor<Dependent>, options?: ProviderOption): {
    execute: <Output>(provider: Provider<Output, Provide>) => Promise<Output>;
    get: () => Promise<Provide>;
    prepare: <Input extends any[], Output_1>(provider: (provide: Provide, ...input: Input) => Output_1) => (...input: Input) => Promise<Awaited<Output_1>>;
    _inject: Hijacked<Dependent>;
};
declare const value: <Provide>(value: Provide) => {
    execute: <Output>(provider: Provider<Output, Provide>) => Promise<Output>;
    get: () => Promise<Provide>;
    prepare: <Input extends any[], Output_1>(provider: (provide: Provide, ...input: Input) => Output_1) => (...input: Input) => Promise<Awaited<Output_1>>;
    _inject: Hijacked<unknown>;
};
declare const execute: <Dependent, Output>(executable: Provider<Output, Dependent>, dep: Executor<Dependent>) => Promise<Output>;
declare const prepare: <Dependent, Input, Output>(provider: (provide: Dependent, input: Input) => Output, dep: Executor<Dependent>) => (input: Input) => Promise<Awaited<Output>>;
type inferProvide<T> = T extends Executor<infer S> ? S : never;
declare const combine: <L extends Record<string, Executor<any>>>(layout: L) => Executor<{ [key in keyof L]: inferProvide<L[key]>; }>;

export { CreateInstrumentHandler, Executor, Hijacked, InstrumentHandler, ProviderOption, combine, composeInstrument, create, createInstrument, defaultProviderOptions, execute, inferProvide, prepare, setInstrument, value };
